-----------------------------------------------------------------------------
-- Split:	Context splitting
-- 
-- Part of `Typing Haskell in Haskell', version of September 1, 1999
-- Copyright (c) Mark P Jones and the Oregon Graduate Institute
-- of Science and Technology, 1999
-- 
-- This program is distributed as Free Software under the terms
-- in the file "License" that is included in the distribution
-- of this software, copies of which may be obtained from:
--             http://www.cse.ogi.edu/~mpj/thih/
-- 
-----------------------------------------------------------------------------

module Split where
import List( (\\), partition )
import Type
import Subst
import Pred

split       :: [Tyvar] -> [Pred] -> ([Pred], [Pred])
split fs     = partition (all (`elem` fs) . tv)
             . simplify []
             . toHnfs

toHnfs      :: [Pred] -> [Pred]
toHnfs       = concat . map toHnf

toHnf       :: Pred -> [Pred]
toHnf p =
 if inHnf p
  then [p]
  else case reducePred p of
         Nothing -> error "context reduction"
         Just ps -> toHnfs ps

inHnf       :: Pred -> Bool
inHnf (IsIn c t) = hnf t
 where hnf (TVar v)  = True
       hnf (TCon tc) = False
       hnf (TAp t _) = hnf t

simplify          :: [Pred] -> [Pred] -> [Pred]
simplify rs []     = rs
simplify rs (p:ps) = simplify (p:(rs\\qs)) (ps\\qs)
 where qs       = bySuper p
       rs \\ qs = [ r | r<-rs, r `notElem` qs ]

-----------------------------------------------------------------------------
